//////////////////////////////////////////////////////////////////////////////
//
//  Sound extension for PocketFrog ver 0.83
//    Written by Thomas 'gersen' Maniero
//
//////////////////////////////////////////////////////////////////////////////

#include <tchar.h>


#include "sound.h"

#include "sound_data.h"



namespace Frog {



Module::Module() {

    m_mh = NULL;
    m_resource = false;
    m_volume = 64;
}

Module::~Module() {

}

void Module::setVolume(uint32_t v) {

    if (v > 64)
        v = 64;

    m_volume = v;
}

uint32_t Module::getVolume() {

    return m_volume;
}

/*bool Module::load(void *p) {
}*/

bool Module::load(HINSTANCE hInstance, DWORD dwResourceID) {

    HRSRC hr; 
    HGLOBAL hg; 

    // trovo risorsa
    hr = FindResource(hInstance, MAKEINTRESOURCE(dwResourceID), TEXT("PTMOD")); 

    if (hr != NULL) { 

        // carico risorsa
        hg = LoadResource(hInstance, hr);

        if (hg != NULL) { 
            // lock resource
            m_mh = (uint8_t *)LockResource(hg); 
            if (m_mh != NULL) {
			    m_resource = true;
                return true;
            }

        }

    }

    return false;
}


SoundFX::SoundFX() {

    m_sample = NULL;
    m_resource = false;
}

SoundFX::~SoundFX() {
}

//bool SoundFX::load(void *p);

void SoundFX::setVolume(uint32_t v) {

    if (v > 64)
        v = 64;

    m_volume = v;
}

uint32_t SoundFX::getVolume() {

    return m_volume;
}


bool SoundFX::load(HINSTANCE hInstance, DWORD dwResourceID) {


    HRSRC hr; 
    HGLOBAL hg; 

    const static int8_t chunkData[4] = {'d', 'a', 't', 'a'};


    // trovo risorsa
    hr = FindResource(hInstance, MAKEINTRESOURCE(dwResourceID), TEXT("WAVE")); 

    if (hr != NULL) { 

        // carico risorsa
        hg = LoadResource(hInstance, hr);

        if (hg != NULL) { 
            // lock resource
            WaveHeader *whdr = (WaveHeader *)((uint32_t)LockResource(hg) + 12);
            uint32_t dist = 0;
            if (whdr != NULL) {
                if ((whdr->nBitsPerSample != 8) || (whdr->nChannels != 1))
                    return false;

                // trovo il chunk data
                while ( (*(uint32_t *)((uint32_t)whdr + dist)) != (*(uint32_t *)(chunkData)) ) {
                    dist += (*(uint32_t *)((uint32_t)whdr + dist + 4)) + 8;
                }

                m_freq = whdr->nSamplesPerSec;
                m_size = (*(uint32_t *)((uint32_t)whdr + dist + 4)) - 8;
                m_volume = 64;

                m_sample = (uint8_t *)((uint32_t)whdr + dist + 8);

                m_resource = true;

                return true;
            }

        }

    }

    return false;
}


// big to little endian 16bit
uint16_t btol16(uint16_t big) {

    uint16_t little;

    ((uint8_t *)&little)[1] = ((uint8_t *)&big)[0];
    ((uint8_t *)&little)[0] = ((uint8_t *)&big)[1];

    return little;
}

// big to little endian 32bit
uint32_t btol32(uint32_t big) {

    uint32_t little;

    ((uint8_t *)&little)[1] = ((uint8_t *)&big)[0];
    ((uint8_t *)&little)[0] = ((uint8_t *)&big)[1];
    ((uint8_t *)&little)[3] = ((uint8_t *)&big)[2];
    ((uint8_t *)&little)[2] = ((uint8_t *)&big)[3];

    return little;
}



Sound::Sound() {

    m_open = false;

    m_bufmix = 0;

    //m_loop = true;
    m_modheader = NULL;
    m_mod = 0;

    for (uint32_t i = 0; i < MOD_MAXCHANNEL; i++) {

        m_vibratoidx[i] = 0;
        m_vibratospeed[i] = 9;
        m_vibratodepth[i] = 2;

        m_slidetonote[i] = 0;
    }
}

Sound::~Sound() {

    close();
}



bool Sound::open(uint32_t freqmix) {

    uint32_t i;

    if (m_open)
        return false;

    switch (freqmix) {
      case 44100 :
          m_bufmixframe = MOD_MIXBUF_44100;
        break;

      case 22050 :
          m_bufmixframe = MOD_MIXBUF_22050;
        break;

      case 11025 :
          m_bufmixframe = MOD_MIXBUF_11025;
        break;

      default :
          m_bufmixframe = MOD_MIXBUF_22050;
          freqmix = 22050;
        break;
    }


    m_freqmix = freqmix;
    m_bufmixtotal = m_bufmixframe*2*(MOD_PPC_WHDR+1);
    m_bufmix = new uint8_t[m_bufmixtotal];
    if (m_bufmix == 0)
        return false;

    // pulisco mixbuffer con scrittura a 8bit
    for (i = 0; i < m_bufmixtotal; i++)
        (*(uint8_t *)((uint32_t)m_bufmix + i)) = 128;

    // nessun mod da suonare
    m_modheader = 0;

    // setto indice del double buffer
    m_dmix = 1;
	m_pbufmix = &m_bufmix[m_dmix*m_bufmixframe*2];

    // setto in pausa
    m_pauseMod = true;
    m_stopMod = false;
    m_pauseSFX = false;

    // setto sfx a NULL
    for (i = 0; i < SFX_MAX; i++)
        m_psfx[i] = NULL;


    // Implementazione PocketPC
    InitializeCriticalSection(&m_critSecRtp);

    m_thread = CreateThread(NULL,           // pointer to security attributes
                          0,                // initial thread stack size
                          update,           // pointer to thread function
                          (void *)this,     // argument for new thread
                          CREATE_SUSPENDED, // creation flags
                          &m_threadId);     // pointer to receive thread ID
    if (m_thread == NULL)
        return false;

    m_killEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    m_exitThread = false;

    MMRESULT res;

    // apro waveout
    m_waveformatx.wFormatTag = WAVE_FORMAT_PCM;
    m_waveformatx.nSamplesPerSec = m_freqmix;
    m_waveformatx.nChannels = 2;
    m_waveformatx.wBitsPerSample = 8;
    m_waveformatx.nBlockAlign = m_waveformatx.nChannels * (m_waveformatx.wBitsPerSample/8);
    m_waveformatx.nAvgBytesPerSec = m_waveformatx.nSamplesPerSec * m_waveformatx.nChannels;
    m_waveformatx.cbSize = 0; // no extra information

    for (i = 0; i < waveOutGetNumDevs(); i++)  {

        res = waveOutOpen(&m_hwaveout,      // LPHWAVEOUT phwo,	
                          i,                // UINT uDeviceID,	
                          &m_waveformatx,   // LPWAVEFORMATEX pwfx,	
                          m_threadId,       // DWORD dwCallback,	      indirizzo proc callback
                          (DWORD)this,      // DWORD dwCallbackInstance, indirizzo ulteriori dati per cb
                          CALLBACK_THREAD); // DWORD fdwOpen	

        if (res == MMSYSERR_NOERROR)
            break;
    }
    if (res != MMSYSERR_NOERROR) {
        CloseHandle(m_killEvent);
        CloseHandle(m_thread);
        delete [] m_bufmix;
        return false;
    }
    

    waveOutPause(m_hwaveout);

    // inserisco nella coda 4 pacchetti vuoti
    for (i = 0; i < MOD_PPC_WHDR; i++) {

        ZeroMemory(&m_wavehdr[i], sizeof(WAVEHDR));
        m_wavehdr[i].dwBufferLength = 8;
        m_wavehdr[i].lpData = (char *)m_bufmix;

        waveOutPrepareHeader(m_hwaveout,       // HWAVEOUT hwo,	
                             &m_wavehdr[i],    // LPWAVEHDR pwh,	
                             sizeof(WAVEHDR)); // UINT cbwh	  

        waveOutWrite(m_hwaveout,       // HWAVEOUT hwo,	
                     &m_wavehdr[i],    // LPWAVEHDR pwh,	
                     sizeof(WAVEHDR)); // UINT cbwh

    }


    // avvio thread sonoro
    SetThreadPriority(m_thread, THREAD_PRIORITY_HIGHEST);
	Sleep(100);
    ResumeThread(m_thread);

    // riavvio waveout
    waveOutRestart(m_hwaveout);


    m_open = true;

    return true;
}


bool Sound::close() {

    if (m_open) {

        // faccio uscire il thread
        m_exitThread = true;

        // aspetto finch esce
        WaitForSingleObject(m_killEvent, 5000);

        CloseHandle(m_killEvent);
        CloseHandle(m_thread);

        DeleteCriticalSection(&m_critSecRtp);

	    waveOutReset(m_hwaveout);

	    for (uint32_t i = 0; i < MOD_PPC_WHDR; i++)
		    waveOutUnprepareHeader(m_hwaveout, &m_wavehdr[i], sizeof(WAVEHDR));

        waveOutClose(m_hwaveout);

        if (m_bufmix)
            delete [] m_bufmix;

        m_open = false;
    }

    return true;
}


uint32_t Sound::playSFX(SoundFX *sndfx, bool loop) {

    uint32_t i;

    if ((!m_open) || (sndfx->m_sample == NULL))
        return 0xFFFF;

    for (i = 0; i < SFX_MAX; i++) {
        if (m_psfx[i] == NULL)
            break;
    }
    if (i == SFX_MAX) {
        //LeaveCriticalSection(&critSecRtp);
        return 0xFFFF;
    }

    EnterCriticalSection(&m_critSecRtp);

    m_psfx[i] = sndfx;
    m_sfxfreqfactor[i] = (m_psfx[i]->m_freq << 16) / (m_freqmix << 8);
    m_sfxposition[i] = 0;
    m_sfxloop[i] = loop;  // set loop

    LeaveCriticalSection(&m_critSecRtp);

    return i;
}


void Sound::stopSFX(uint32_t channel) {

    if (!m_open)
        return;

    EnterCriticalSection(&m_critSecRtp);

    if (channel < SFX_MAX)
        m_psfx[channel] = NULL;

    LeaveCriticalSection(&m_critSecRtp);
}



// estraggo dalla sequenza pattern il numero maggiore
// ottengo quindi il numero di pattern presenti
uint32_t snd_getNumPattern(ModHeader *mh) {

    uint32_t i, max;

    max = 0;
    for (i = 0; i < mh->numSequence; i++) {
        if (max < mh->sequences[i])
            max = mh->sequences[i];
    }

    return max+1;
}

void Sound::playMod(Module *mod, bool loop) {

    uint32_t start, now, i;

    if ((!m_open) || (mod->m_mh == NULL))
        return;

    m_mod = mod;

    EnterCriticalSection(&m_critSecRtp);

    // salvo puntatore
    m_modheader = (ModHeader *)m_mod->m_mh;

    m_modpattern = (ModPattern *)((uint32_t)m_modheader + sizeof(ModHeader));

    // calcolo dati sample.. 1..31 ma tengo lo 0 per facilitarmi
    // successivamente la gestione
    start = (uint32_t)m_modpattern + sizeof(ModPattern) * snd_getNumPattern(m_modheader);
    now = 0;

    // setto a 0 per evitare di processare sample usati per il silenzio
    m_sdata[0] = 0;
    m_sfinetune[0] = 0;
    for (i = 1; i < MOD_MAXSAMPLE; i++) {
        if (m_modheader->samples[i-1].length != 0) {

            m_slength[i] = btol16(m_modheader->samples[i-1].length) << 9; // *2 (in word...) e fixo
            m_sloopstart[i] = btol16(m_modheader->samples[i-1].loopstart) << 9;
            if (btol16(m_modheader->samples[i-1].looplength) <= 1)
                m_slooplength[i] = 0;
            else
                m_slooplength[i] = btol16(m_modheader->samples[i-1].looplength) << 9;

            m_svolume[i] = m_modheader->samples[i-1].volume;
            m_sfinetune[i] = (uint32_t)m_modheader->samples[i].finetune;
            

            m_sdata[i] = (start + now);
            now += btol16(m_modheader->samples[i-1].length)*2;
        }
        else
            m_sdata[i] = 0;       
    }


    // setto dati di default

    m_bytextick = m_freqmix/(125*2/5);
    m_bytecnt = 0;

    // setto anche tickcnt in modo che scatti immediatamente e mi legga le prime 4 note
    m_tickxdivision = 6;
    m_tickcnt = m_tickxdivision;

    m_seq = 0;
    m_pattern = m_modheader->sequences[m_seq];
    m_note = 0*4;

    m_jumpseq = m_jumpnote = 0xFFFFFF;

    m_delaypattern = 0;

    // setto default pan
    m_pan[0] = m_pan[3] = 64;
    m_pan[1] = m_pan[2] = 192;
   
    // pulisco dati essenziali
    for (i = 0; i < MOD_MAXCHANNEL; i++) {
        m_sample[i] = 0;
        m_notefreq[i] = 0;
    }

    m_pauseMod = false;
    m_stopMod = false;

    // set loop
    m_loop = loop;
    
    LeaveCriticalSection(&m_critSecRtp);
}




#define from 0
#define to 4


uint32_t ext_sample;
uint32_t ext_command, ext_x, ext_y, ext_xy;
uint32_t ext_notefreq;


void extractNote(uint32_t n) {

    ext_command = (n & 0x000F0000) >> 16;
    ext_x = n >> 28;
    ext_y = (n & 0x0F000000) >> 24;
    ext_xy = (n & 0xFF000000) >> 24;

    ext_sample = ((n & 0x00F00000) >> 20) | (n & 0xF0);

    ext_notefreq = ((n & 0xF) << 8) | ((n & 0xFF00) >> 8);
}


void Sound::updateSFX(uint32_t start) {

    uint32_t i, p;
    int32_t l, r, s;

    p = start;
    while (p < m_bufmixframe*2) {

        l = r = 0;
        for (i = 0; i < SFX_MAX; i++) {
            if (m_psfx[i] != NULL) {

		        if (m_sfxloop[i]) {
		            if ( m_sfxposition[i] > (m_psfx[i]->m_size << 8) )
			            m_sfxposition[i] -= (m_psfx[i]->m_size << 8);
                }

                if ( m_sfxposition[i]  <= (m_psfx[i]->m_size << 8) ) {
                    s = (*(uint8_t *)( (uint32_t)m_psfx[i]->m_sample + (m_sfxposition[i] >> 8)));
                    s -= 128;

                    s *= m_psfx[i]->m_volume;

                    l += s;
                    r += s;

                    m_sfxposition[i] += m_sfxfreqfactor[i];
                }
                else {
                    m_psfx[i] = NULL;
                }
            }

        }

        l >>= 7;
        r >>= 7;


        if (l > 127)
            l = 127;
        if (r > 127)
            r = 127;
        if (l < -128)
            l = -128;
        if (r < -128)
            r = -128;

        l += 128;
        r += 128;

        m_pbufmix[p] = (uint8_t)(l);
        m_pbufmix[p + 1] = (uint8_t)(r);
        p += 2;
    }
}

void Sound::updateModSFX() {

    uint32_t i, p;
    int32_t l, r, s;
    uint32_t freqsample;



    p = 0;
    while (p < m_bufmixframe*2) {

        if (m_tickcnt == m_tickxdivision) {

            if (m_delaypattern == 0) {

               // nuove note
                for (i = from; i < to; i++) {

                    // prendo la nuova nota
                    uint32_t n = m_modpattern[m_pattern].notes[i + m_note].bits;
                    //extractNote(modpattern[pattern].notes[i + note].bits);
                    ext_command = (n & 0x000F0000) >> 16;
                    ext_x = n >> 28;
                    ext_y = (n & 0x0F000000) >> 24;
                    ext_xy = (n & 0xFF000000) >> 24;
                    ext_sample = ((n & 0x00F00000) >> 20) | (n & 0xF0);
                    ext_notefreq = ((n & 0xF) << 8) | ((n & 0xFF00) >> 8);

                    // estraggo dati comandi
                    m_command[i] = ext_command;
                    m_opx[i] = ext_x; m_opy[i] = ext_y;
                    m_opxy[i] = ext_xy;


                    // resetta volume e update sample
                    if (ext_sample != 0) {
                        m_sample[i] = ext_sample;
                        m_volume[i] = m_svolume[ext_sample];
                        m_gvolume[i] = (m_volume[i] * m_mod->m_volume) >> 6;
                    }

                    // resetta posizione e update notefreq (no se effect 3, 5, ED)
                    if (ext_notefreq != 0) {
                        if ((m_command[i] != 0x03) && (m_command[i] != 0x05) && ((m_command[i]<<4)+m_opx[i] != 0xED)) {
                            m_position[i] = 0;
                            m_notefreq[i] = amiganote[noteidx[ext_notefreq] + 60*m_sfinetune[m_sample[i]] ];
                        }
                    }


                    // se ho un sample nel canale...
                    if (m_sample[i] != 0) {
                        freqsample = samplefreq[m_notefreq[i]]; // * finetune
                        m_freqfactor[i] = (freqsample << 8) / (m_freqmix << 8);
                    }


                    switch (m_command[i]) {

                        // effetto 0x00 Arpeggio
                        case 0x00 : {

                            if (m_opxy[i] == 0)
                                break;

                            m_arpeggioidx[i] = 0;
                            m_arpeggiofq[i] = m_freqfactor[i];
           
                            m_arpeggiofqx[i] = (samplefreq[amiganote[noteidx[m_notefreq[i]] + m_opx[i] ]] << 8) / (m_freqmix << 8);
                            m_arpeggiofqy[i] = (samplefreq[amiganote[noteidx[m_notefreq[i]] + m_opy[i] ]] << 8) / (m_freqmix << 8);                     
                        

                            break;
                        }

                        // effetto 0x0A (10) volume slide 
                        // effetto 0x06  vibrato + vol slide
                        // effetto 0x05  vibrato + vol slide
                        case 0x05 :
                        case 0x06 :
                        case 0x0A :

                            if (m_opxy[i] == 0) {
                                m_voldelta[i] = 0;
                                break;
                            }
    
                            if (m_opx[i] != 0)
                                m_voldelta[i] = m_opx[i];
                            else 
                                m_voldelta[i] = -(int32_t)m_opy[i];

                        break;
                    

                        // effetto 0x01 (1) slide (portamento) up
                        case 0x01 : {
                            m_slideupdown[i] = m_notefreq[i];
                            break;
                        }

                        // effetto 0x02 (2) slide (portamento) down
                        case 0x02 : {
                            m_slideupdown[i] = m_notefreq[i];
                            break;
                        }

                        // effetto 0x03 (3) slide to note
                        case 0x03 : {

                            if (ext_notefreq != 0) {
                                m_slidetonote[i] = amiganote[noteidx[ext_notefreq] + 60*m_sfinetune[m_sample[i]] ];
                            }
                            else {
                                if (m_slidetonote[i] == 0)
                                    m_slidetonote[i] = amiganote[noteidx[m_notefreq[i]] + 60*m_sfinetune[m_sample[i]] ];
                            }

                            if (m_opxy[i] != 0)
                                m_slidetonotedelta[i] = m_opxy[i];

                            break;
                        }

                        // effetto 0x04 (4) vibrato
                        case 0x04 : {

                            if (m_opxy[i] == 0) {
                                break;
                            }

                            m_vibratospeed[i] = m_opx[i];
                            m_vibratodepth[i] = m_opy[i];

                            break;
                        }

                        case 0x09 : {

                            m_position[i] = (m_opx[i] << 12) + (m_opy[i] << 8);

                            break;
                        }
                    

                        // effetto 0xC () set volume
                        case 0x0C : {
                            m_volume[i] = m_opxy[i];
                            m_gvolume[i] = (m_volume[i] * m_mod->m_volume) >> 6;
                            break;
                        }

                        // effetto 0x0F (15) set vel/tempo (per ora solo tick/division)
                        case 0x0F : {
                            if (m_opxy[i] <= 32)
                                m_tickxdivision = m_opxy[i];
                            else
                                m_bytextick = m_freqmix / (m_opxy[i]*2 / 5);

                            break;
                        }


                        // pattern jump (10)
                        case 0x0B : {
                            m_jumpseq = m_opxy[i];
                            m_jumpnote = 0;
                            break;
                        }

                        // comando pattern break (13)
                        case 0x0D : {
                            // se il prossimo nella sequenza non esiste
                            if ((m_seq+1) >= m_modheader->numSequence) {
                                // se ho il loop riparto
                                if (m_loop) {
                                    m_jumpseq = 0;
                                    m_jumpnote = (m_opx[i]*10+m_opy[i]) << 2;
                                }
                                // altrimenti vado alla fine
                                else {
                                    m_jumpnote = 63 << 2;
                                }
                      
                            }
                            // altrimenti passo al prossimo nella sequenza
                            else {
                                m_jumpseq = m_seq+1;
                                m_jumpnote = (m_opx[i]*10+m_opy[i]) << 2;
                            }
                            
                            break;
                        }


                        // exctended effect
                        case 0x0E : {

                            switch (m_opx[i]) {

                                // effect 0xED note delay o delay sample (13)
                                case 0xD : {
                                    m_delaynote[i] = m_opy[i]-1;
                                    m_delaynotefreq[i] = ext_notefreq;
                                    break;
                                }

                                // effect 0xEE pattern delay
                                case 0xE : {
                                    m_delaypattern = m_opy[i];
                                    break;
                                }

                                // effect 0xEA fine volume slide up
                                case 0xA : {
                                    m_volume[i] += m_opy[i];
                                    m_gvolume[i] = (m_volume[i] * m_mod->m_volume) >> 6;
                                    break;
                                }

                                // effect 0xEB fine volume slide down
                                case 0xB : {
                                    m_volume[i] -= m_opy[i];
                                    m_gvolume[i] = (m_volume[i] * m_mod->m_volume) >> 6;
                                    break;
                                }

                            } // switch
                        }
                
                
                    } // switch

                }


                // aggiorno note, pattern, sequenza
                if (m_jumpseq != 0xFFFFFF) {
                    m_seq = m_jumpseq;
                    m_pattern = m_modheader->sequences[m_seq];
                    m_note = m_jumpnote;
                    m_jumpseq = 0xFFFFFF;
                }
                else {
                    m_note += 4;
                    if (m_note == 256) {
                        m_note = 0;
                        m_seq++;
                        if (m_seq >= m_modheader->numSequence) {
                            if (m_loop) {
                                m_seq = 0;
                            }
                            // stoppo
                            else {
                                // metto nel caso si rifacesse partire... ???
                                m_seq = 0;
                                m_stopMod = true;

                                // scrivo gli ultimi eventuali byte
                                if (m_pauseSFX == false) {
                                    updateSFX(p);
                                }
                                else {
                                    while (p < m_bufmixframe*2) {
                                        m_pbufmix[p] = 128;
                                        m_pbufmix[p + 1] = 128;
                                        p += 2;
                                    }
                                }

                                return;
                            }
                        }

                        m_pattern = m_modheader->sequences[m_seq];
                    }
                }


            } // if delaypattern != 0
            else {
                m_delaypattern--;
            }


            m_tickcnt = 0;
        }
        else {

            if (m_bytecnt == m_bytextick) {
                m_tickcnt++;
                m_bytecnt = 0;

                if (m_tickcnt != m_tickxdivision) {

                // update dell'effetto... (gli effetti sono tick dipendenti)
                for (i = from; i < to; i++) {

                    
                    switch (m_command[i]) {
 
                        // effetto 0x00 Arpeggio
                        case 0x00 : {
                            // se 0 no arpeggio in pratica
                            if (m_opxy[i] == 0)
                                break;

                            m_arpeggioidx[i]++;
                            if (m_arpeggioidx[i] == 3)
                                m_arpeggioidx[i] = 0;
                            switch (m_arpeggioidx[i]) {
                              case 0:
                                  m_freqfactor[i] = m_arpeggiofq[i];
                                  break;
                              case 1:
                                  m_freqfactor[i] = m_arpeggiofqx[i];
                                  break;
                              case 2:
                                  m_freqfactor[i] = m_arpeggiofqy[i];
                                  break;
                            }

                            break;
                        }

   
                        // effetto 0x01 (1) slide (portamento) up
                        case 0x01 : {
                            m_slideupdown[i] -= m_opxy[i];
                            freqsample = samplefreq[m_slideupdown[i]]; // * finetune
                            if (freqsample != 0)
                                m_freqfactor[i] = (freqsample << 8) / (m_freqmix << 8);

                            m_notefreq[i] = m_slideupdown[i];

                            break;
                        }

                        // effetto 0x02 (2) slide (portamento) down
                        case 0x02 : {
                            m_slideupdown[i] += m_opxy[i];
                            freqsample = samplefreq[m_slideupdown[i]]; // * finetune
                            if (freqsample != 0)
                                m_freqfactor[i] = (freqsample << 8) / (m_freqmix << 8);

                            m_notefreq[i] = m_slideupdown[i];

                            break;
                        }

                        // effetto 0x03 (3) slide to note
                        case 0x05 :
                        case 0x03 :
                            //uint32_t refnote = actual[i].notefreq;
                            if (m_notefreq[i] > m_slidetonote[i]) {
                                m_notefreq[i] -= m_slidetonotedelta[i];
                                if ((m_notefreq[i] < m_slidetonote[i]) || ((int32_t)m_notefreq[i] < 0))
                                    m_notefreq[i] = m_slidetonote[i];
                            }

                            if (m_notefreq[i] < m_slidetonote[i]) {
                                m_notefreq[i] += m_slidetonotedelta[i];
                                if ((m_notefreq[i] > m_slidetonote[i]))
                                    m_notefreq[i] = m_slidetonote[i];
                            }

                            freqsample = samplefreq[m_notefreq[i]]; // * finetune
                            m_freqfactor[i] = (freqsample << 8) / (m_freqmix << 8);

                            // anche volume slide
                            if (m_command[i] == 0x05) {
                                m_volume[i] += m_voldelta[i];
                                if ((int32_t)m_volume[i] < 0)
                                    m_volume[i] = 0;
                                if (m_volume[i] > 64)
                                    m_volume[i] = 64;
                                m_gvolume[i] = (m_volume[i] * m_mod->m_volume) >> 6;
                            }

                          break;
                        

                        // effetto 0x04 (4) vibrato
                        case 0x06 :
                        case 0x04 :

                            if (m_vibratoidx[i] >= 32)
                                freqsample = samplefreq[((m_notefreq[i] << 8) - (sine[m_vibratoidx[i]-32] * (m_vibratodepth[i]))) >> 8];
                            else
                                freqsample = samplefreq[((m_notefreq[i] << 8) + (sine[m_vibratoidx[i]] * (m_vibratodepth[i]))) >> 8];

                            m_vibratoidx[i] += m_vibratospeed[i];
                            if (m_vibratoidx[i] >= 64)
                                m_vibratoidx[i] -= 64;

                            m_freqfactor[i] = (freqsample << 8) / (m_freqmix << 8);

                            // anche volume slide
                            if (m_command[i] == 0x06) {
                                m_volume[i] += m_voldelta[i];
                                if ((int32_t)m_volume[i] < 0)
                                    m_volume[i] = 0;
                                if (m_volume[i] > 64)
                                    m_volume[i] = 64;
                                m_gvolume[i] = (m_volume[i] * m_mod->m_volume) >> 6;
                            }
         
                          break;
                        
                        
                        // effetto 0x0A (10) volume slide
                        case 0x0A : {
                            m_volume[i] += m_voldelta[i];
                            if ((int32_t)m_volume[i] < 0)
                                m_volume[i] = 0;
                            if (m_volume[i] > 64)
                                m_volume[i] = 64;
                            m_gvolume[i] = (m_volume[i] * m_mod->m_volume) >> 6;

                            break;
                        }


                        // extended effect
                        case 0x0E : {
 
                            switch (m_opx[i]) {

                                // effect 0xED note delay o delay sample (13)
                                case 0xD : {

                                    if (m_delaynote[i] == 0xFFFFFFFF) {
                                        m_notefreq[i] = m_delaynotefreq[i];
                                        m_delaynote[i] = 0x00FFFFFF;
                                        freqsample = samplefreq[m_notefreq[i]]; // * finetune
                                        m_freqfactor[i] = (freqsample << 8) / (m_freqmix << 8);
                                    }
                                    else {
                                        if (m_delaynote[i] != 0x00FFFFFF)
                                            m_delaynote[i]--;
                                    }

                                }

                            } // switch
                        }

                        

                    } // switch


                } // for

                } // l'if riparatore :)

            }
            else
                m_bytecnt++;
        }


        l = r = 0;
        for (i = from; i < to; i++) {

            uint32_t p;

            //if (m_sample[i] != 0) {
            if (m_sdata[m_sample[i]] != 0) {
                if (m_slooplength[m_sample[i]] == 0) {
                    if (m_position[i] <= m_slength[m_sample[i]]) {

                        s = (*(int8_t *)( m_sdata[m_sample[i]] + (m_position[i] >> 8)));

                        p = (m_pan[i] * m_gvolume[i]) >> 8;
                        l += (m_gvolume[i] - p) * s;
                        r += p * s;
                    }

                    m_position[i] += m_freqfactor[i];
                }
                // with loop
                else {             
                    if (m_position[i] > (m_sloopstart[m_sample[i]] + m_slooplength[m_sample[i]]) )
                        m_position[i] -= m_slooplength[m_sample[i]];
                        
                    s = (*(int8_t *)( m_sdata[m_sample[i]] + (m_position[i] >> 8)));

                    p = (m_pan[i] * m_gvolume[i]) >> 8;
                    l += (m_gvolume[i] - p) * s;
                    r += p * s;

                    m_position[i] += m_freqfactor[i];
                }
            }

        }


        if (m_pauseSFX == false) {
            for (i = 0; i < SFX_MAX; i++) {
                if (m_psfx[i] != NULL) {

                    if (m_sfxloop[i]) {
                        if ( m_sfxposition[i] > (m_psfx[i]->m_size << 8) )
                            m_sfxposition[i] -= (m_psfx[i]->m_size << 8);
                    }

                    if ( m_sfxposition[i]  <= (m_psfx[i]->m_size << 8) ) {
                        s = (*(uint8_t *)( (uint32_t)m_psfx[i]->m_sample + (m_sfxposition[i] >> 8))) /* psfx[i]->volume*/;
                        s -= 128;

                        s *= m_psfx[i]->m_volume;

                        l += s;
                        r += s;

                        m_sfxposition[i] += m_sfxfreqfactor[i];
                    }
                    else {
                        m_psfx[i] = NULL;
                    }
                }
            }
        }

        l >>= 7;
        r >>= 7;


        if (l > 127)
            l = 127;
        if (r > 127)
            r = 127;
        if (l < -128)
            l = -128;
        if (r < -128)
            r = -128;

        l += 128;
        r += 128;

        m_pbufmix[p] = (uint8_t)(l);
        m_pbufmix[p + 1] = (uint8_t)(r);
        p += 2;
    }

    //return note;
}



DWORD WINAPI Sound::update(LPVOID lpParameter) {


    Sound *snd;
    MSG msg;
    WAVEHDR *pwavehdr;
    uint32_t i;
	//DWORD dwWait;


    snd = (Sound *)lpParameter;
	if (snd == NULL)
	    return -1;


    //while (!snd->m_exitThread) {
	while ((WaitForSingleObject(snd->m_killEvent, 5) != 0) && (!snd->m_exitThread)) {


        while (PeekMessage(&msg, 0, 0, 0, PM_REMOVE)) {

            if (msg.message == MM_WOM_DONE) {  
				
                pwavehdr = (WAVEHDR *)msg.lParam;
			    if (pwavehdr == NULL)
				    break;

				
                waveOutUnprepareHeader(snd->m_hwaveout, pwavehdr, sizeof(WAVEHDR));

                ZeroMemory(pwavehdr, sizeof(WAVEHDR));
                pwavehdr->dwBufferLength = snd->m_bufmixframe*2;
                pwavehdr->lpData = (char *)snd->m_pbufmix;

                waveOutPrepareHeader(snd->m_hwaveout, pwavehdr, sizeof(WAVEHDR));

                waveOutWrite(snd->m_hwaveout, pwavehdr, sizeof(WAVEHDR));


                snd->m_dmix++;
                if (snd->m_dmix >= (MOD_PPC_WHDR+1))
                    snd->m_dmix = 0;

                snd->m_pbufmix = &snd->m_bufmix[snd->m_dmix*snd->m_bufmixframe*2];

                EnterCriticalSection(&snd->m_critSecRtp);
                // se musica in pausa o fermo azzero buffer
                if ((snd->m_pauseMod) || (snd->m_stopMod)) {

                    // processo solo sfx
                    if (snd->m_pauseSFX == false) {
                        snd->updateSFX();
                    }
                    else {
                        // prossimo buffer a 0
                        for (i = 0; i < snd->m_bufmixframe*2; i++) {
                            (*(uint8_t *)((uint32_t)snd->m_pbufmix + i)) = 128;
                            //(*(uint32_t *)((uint32_t)pbufmix + (MOD_MIXBUF*2) + (i << 2))) = 0;
                        }
                    }

                }
                else {
                    // processo tutto...
                    snd->updateModSFX();
                }

                LeaveCriticalSection(&snd->m_critSecRtp);

            }// if (msg.message == MM_WOM_DONE)

        }// while peekmessage
        
    }// while (1)


    // setto evento
    SetEvent(snd->m_killEvent);
    return 0;
}



void Sound::pauseSFX() {

    if (!m_open)
        return;

    EnterCriticalSection(&m_critSecRtp);

    m_pauseSFX = true;
    
    LeaveCriticalSection(&m_critSecRtp);
}

void Sound::resumeSFX() {

    if (!m_open)
        return;

    EnterCriticalSection(&m_critSecRtp);

    m_pauseSFX = false;
    
    LeaveCriticalSection(&m_critSecRtp);
}


void Sound::stopMod() {

    if (!m_open)
        return;

    EnterCriticalSection(&m_critSecRtp);

    m_stopMod = true;    
    // m_modheader = 0 ??? in teoria dovrebbe bastare per lo stop!!!

    LeaveCriticalSection(&m_critSecRtp);
}

void Sound::pauseMod() {

    if (!m_open)
        return;

    EnterCriticalSection(&m_critSecRtp);

    if (m_modheader != 0)
        m_pauseMod = true;

    LeaveCriticalSection(&m_critSecRtp);
}

void Sound::resumeMod() {

    if (!m_open)
        return;

    EnterCriticalSection(&m_critSecRtp);

    if (m_modheader != 0)
        m_pauseMod = false;
    
    LeaveCriticalSection(&m_critSecRtp);
}


void Sound::suspend() {

    if (!m_open)
        return;

    /*pauseSFX();
    pauseMod();*/
    waveOutPause(m_hwaveout);
    SuspendThread(m_thread);
}

void Sound::resume() {

    if (!m_open)
        return;

    /*resumeSFX();
    resumeMod();*/
    ResumeThread(m_thread);
    waveOutRestart(m_hwaveout);
}


}
