//////////////////////////////////////////////////////////////////////////////
//
//  Sound extension for PocketFrog ver 0.83
//    Written by Thomas 'gersen' Maniero
//
//////////////////////////////////////////////////////////////////////////////


#ifndef _SOUND_H
#define _SOUND_H


#include <windows.h>
#include <mmreg.h>

#include "defs.h"




#define MOD_MAXCHANNEL 4
#define MOD_MAXSAMPLE 32

#define SFX_MAX 8

#define MOD_MIXBUF_11025 492
#define MOD_MIXBUF_22050 984  //484
#define MOD_MIXBUF_44100 1968

#define MOD_PPC_WHDR 4



namespace Frog {


// ATTENZIONE tutti i word vanno capovolti... l'amiga  una macchina big-endian


// struttura per i mod sample... 
// sono 15 o 31. se 31 dopo i primi 15 NON abbiamo la stringa di 4 byte "M.K."
struct struct_ModSample {
    int8_t name[22];       // nome del sample
    uint16_t length;        // lunghezza in word (va quindi moltiplicato per 2) (* 2)
    int8_t finetune;       // nota... lower 4 bits (-7..8)
    uint8_t volume;         // 0..64 (64? dovrebbe essere 63) -- vol = 20*log10(Vol/64)
    uint16_t loopstart;     // offset in word inizio del sample loop (* 2)
    uint16_t looplength;    // lunghezza in word del sample loop (loop se > 1) (* 2)
};
typedef struct struct_ModSample ModSample;


union union_ModNote {
    struct {
        unsigned notefreq : 12;
        unsigned : 4;
        unsigned y : 4;
        unsigned x : 4;    
        unsigned command : 4;
        unsigned sample : 4;    // 0 = NULL, 1..31 valid
    };
    struct {
        unsigned : 16;
        unsigned xy : 8;
        unsigned : 8;
    };
    uint32_t bits;
};
typedef union union_ModNote ModNote;


struct struct_ModPattern {
    ModNote notes[256];
    //ModNote notes[64][4];
};
typedef struct struct_ModPattern ModPattern;




// module header
struct struct_ModHeader {
    int8_t name[20];           // nome del modulo
    ModSample samples[31]; // sample del modulo
    uint8_t numSequence;        // numero sequenze (?) (1..128)
    uint8_t restart;            // non pi usato (?)
    uint8_t sequences[128];     // sequenza (numSequence) suono pattern (ogni byte compreso 0..63)
    int8_t mkSign[4];          // "M.K." signature

    // pattern data...
    // ModPattern *patterns; // array di pattern, contiene 1024 note divise per 64

    // sample data...

};
typedef struct struct_ModHeader ModHeader;


typedef struct {
    int8_t ckID[4];          // "fmt " signature
    uint32_t nChunkSize;      // lunghezza di questo chunk (compreso ckID)
    uint16_t wFormatTag;      // formato del wave
    uint16_t nChannels;       // numero canali
    uint32_t nSamplesPerSec;  // sample rate (11025 - 22050 - 44100...)
    uint32_t nAvgBytesPerSec;
    uint16_t nBlockAlign;     
    uint16_t nBitsPerSample;  // bit campionamento... (8 - 16)

} WaveHeader;



class Module {

    friend class Sound;

private:
    uint8_t *m_mh;
    bool m_resource;
    uint32_t m_volume;

public:
    Module();
    ~Module();

    void setVolume(uint32_t v);
    uint32_t getVolume();

    //bool load(void *p);
    bool load(HINSTANCE hInstance, DWORD dwResourceID);
};


// struttura per gli effetti sonori
class SoundFX {   

    friend class Sound;

private:
    uint8_t *m_sample;        // puntatore ai dati

    uint32_t m_freq;          // frequenza campionamento
    uint32_t m_size;          // lunghezza in byte del suono
    uint32_t m_left;          // 1 - canale sinistro, 0 - canale destro
    uint32_t m_volume;        // volume suono

    bool m_resource;

public:
    SoundFX();
    ~SoundFX();

    void setVolume(uint32_t v);
    uint32_t getVolume();

    //bool load(void *p);
    bool load(HINSTANCE hInstance, DWORD dwResourceID);
};



class Sound {

private:
    HWAVEOUT m_hwaveout;
    WAVEFORMATEX m_waveformatx;
    WAVEHDR m_wavehdr[MOD_PPC_WHDR];

    HANDLE m_thread;
    DWORD m_threadId;
    CRITICAL_SECTION m_critSecRtp;
    HANDLE m_killEvent;
    bool m_exitThread;

    bool m_open;

    Module *m_mod;


    // contiene i puntatori degli effetti da suonare
    SoundFX *m_psfx[SFX_MAX];
    uint32_t m_sfxfreqfactor[SFX_MAX];
    uint32_t m_sfxposition[SFX_MAX];
    bool m_sfxloop[SFX_MAX];

    // buffer mix
    uint8_t *m_bufmix;
    // lunghezza di un frame (senza il doppio per lo stereo)
    uint32_t m_bufmixframe;
    // totale lunghezza bufmix
    uint32_t m_bufmixtotal;
    // puntatore al buffer precedente (per il double buffer)
    uint8_t *m_pbufmix;
    // indice per double buffer
    uint32_t m_dmix;
    // dati attuale modulo
    ModHeader *m_modheader;
    ModPattern *m_modpattern;
    // attuale pattern e note e numero nella sequenza pattern
    uint32_t m_pattern, m_note, m_seq;
    // dati frequenze e tempo
    uint32_t m_freqmix;
    uint32_t m_bytextick, m_bytecnt;
    uint32_t m_tickxdivision, m_tickcnt;
    // dati per note e comandi canali
    uint32_t m_notefreq[MOD_MAXCHANNEL];
    uint32_t m_sample[MOD_MAXCHANNEL];
    uint32_t m_freqfactor[MOD_MAXCHANNEL];
    uint32_t m_volume[MOD_MAXCHANNEL];      // mi serve per i calcoli del vol slide
    uint32_t m_gvolume[MOD_MAXCHANNEL];     // gvolume contiene il vol modificato dal vol del m_mod
    uint32_t m_command[MOD_MAXCHANNEL];
    uint32_t m_opx[MOD_MAXCHANNEL];
    uint32_t m_opy[MOD_MAXCHANNEL];
    uint32_t m_opxy[MOD_MAXCHANNEL];
    uint32_t m_position[MOD_MAXCHANNEL];
    uint32_t m_pan[MOD_MAXCHANNEL];
    // dati per sample
    uint32_t m_slength[MOD_MAXSAMPLE];
    uint32_t m_sloopstart[MOD_MAXSAMPLE];
    uint32_t m_slooplength[MOD_MAXSAMPLE];
    uint32_t m_svolume[MOD_MAXSAMPLE];
    uint32_t m_sfinetune[MOD_MAXSAMPLE];
    uint32_t m_sdata[MOD_MAXSAMPLE];
    // dati per effetti
    int32_t m_voldelta[MOD_MAXCHANNEL];
    uint32_t m_slideupdown[MOD_MAXCHANNEL];
    uint32_t m_slidetonote[MOD_MAXCHANNEL];
    uint32_t m_slidetonotedelta[MOD_MAXCHANNEL];
    uint32_t m_arpeggioidx[MOD_MAXCHANNEL];
    uint32_t m_arpeggiofq[MOD_MAXCHANNEL];   // freqfactor note
    uint32_t m_arpeggiofqx[MOD_MAXCHANNEL];  // freqfactor note+x
    uint32_t m_arpeggiofqy[MOD_MAXCHANNEL];  // freqfactor note+y
    uint32_t m_vibratoidx[MOD_MAXCHANNEL];   // indice alla tabella
    uint32_t m_vibratospeed[MOD_MAXCHANNEL]; // adder all'indice
    uint32_t m_vibratodepth[MOD_MAXCHANNEL]; // profondit curva, da moltiplicare al valore tabella
    uint32_t m_delaynote[MOD_MAXCHANNEL];
    uint32_t m_delaynotefreq[MOD_MAXCHANNEL];
    uint32_t m_jumpseq;
    uint32_t m_jumpnote;
    uint32_t m_delaypattern;

    bool m_pauseMod, m_stopMod, m_pauseSFX;
    bool m_loop;


private:
    static DWORD WINAPI update(LPVOID lpParameter);
    void updateSFX(uint32_t start = 0);
    void updateModSFX();


public:
    Sound();
    ~Sound();

    bool open(uint32_t freqmix);
    bool close();

    void suspend();
    void resume();

    uint32_t playSFX(SoundFX *sndfx, bool loop = false);
    void stopSFX(uint32_t channel);
    void pauseSFX();
    void resumeSFX();

    void playMod(Module *mod, bool loop = false);
    void stopMod();
    void pauseMod();
    void resumeMod();
};


}



#endif
